#include <stdio.h>
#include <sys/types.h>
#include <stdbool.h>
#include <getopt.h>
#include <libusb-1.0/libusb.h>
//#pragma comment(lib "libusb-1.0.a")

libusb_device_handle *devh=NULL;
unsigned short addr=0;
unsigned short data=0;
bool isWrite = false;

static struct option opts[] = {
  	{"device", 1, 0, 'd'},
	{"addr", 1, 0, 256},
	{"data", 1, 0, 257},
	{"read", 1, 0, 'r'},
	{"write", 1, 0, 'w'},
  	{0,0,0,0}
};

static void usage(const char *argv0)
{
	printf("Usage: %s [options] device\n", argv0);
	printf("Supported options:\n");
	printf("-d, 		device(/dev/video0)\n");
	printf("-r, --read	read DSP.\n");
	printf("-w, --write	write DSP.\n");
	printf("-h, --help	Show this help screen\n");
	printf("--addr	Set the DSP address.\n");
	printf("--data	Set the DSP data.\n");
	
}

static void print_devs(libusb_device **devs)
{
	libusb_device *dev;
	int ret = -1;
	int i = 0;

	//unsigned char *data;

	while ((dev = devs[i++]) != NULL) {
		struct libusb_device_descriptor desc;
		int r = libusb_get_device_descriptor(dev, &desc);
		if (r < 0) {
			fprintf(stderr, "failed to get device descriptor");
			return;
		}
		
		if (libusb_get_device_address(dev) > 1)
		{
			devh = NULL;
			if (desc.idVendor == 0x0c45)
			{
				devh = libusb_open_device_with_vid_pid(NULL, desc.idVendor, desc.idProduct);	
				if (NULL == devh)
					continue;
				
				ret = libusb_attach_kernel_driver(devh, 0);		
				
				if (isWrite)
					ret = libusb_control_transfer(devh,
	0x41, 0x08, addr, 0x0c45, 
	&data, 1, 500);
				else
					ret = libusb_control_transfer(devh,
	0xc1, 0x00, addr, 0x0c45, 
	&data, 1, 500);

printf("%04x:%04x (bus %d, device %d) 0x1006: %x \n",
			desc.idVendor, desc.idProduct,
			libusb_get_bus_number(dev), libusb_get_device_address(dev), data);
			}
			
		}
		
	}
}

int main(int argc, char* argv[])
{
	libusb_device **devs;
	int r;
	ssize_t cnt;

	int ret, nameid=0;		

	while ((ret=getopt_long(argc, argv, "d:rw",opts,NULL))!=-1)
    	{
      	switch(ret)
	{
	case 'd':
	  	nameid=optind-1;
	  	break;
	case 'r':
		break;
	case 'w':
		isWrite = true;
		break;
	case 'h':
		usage(argv[0]);
		return 0;
	case 256:
		addr = strtol(optarg, NULL, 16); //atoi(optarg);
		break;
	case 257:
		data = strtol(optarg, NULL, 16);//atoi(optarg);
		break;
	default:
		usage(argv[0]);		
	  return 1;
	}
	}
	
	

	r = libusb_init(NULL);
	if (r < 0)
		return r;
	
	cnt = libusb_get_device_list(NULL, &devs);
	if (cnt < 0)
		return (int)cnt;

	print_devs(devs);
	libusb_detach_kernel_driver(devh, 0);
	libusb_free_device_list(devs, 1);

	libusb_exit(NULL);
	return 0;
}
